@@ -1,7 +1,9 @@
 /** @file
  * @brief Parser for OpenDocument's meta.xml.
+ *
+ * Also used for MSXML's docProps/core.xml.
  */
-/* Copyright (C) 2006,2009,2010,2011,2016 Olly Betts
+/* Copyright (C) 2006,2009,2010,2011,2016,2022 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,14 +28,15 @@
 #include <ctime>
 
 class MetaXmlParser : public HtmlParser {
-    enum { NONE, KEYWORDS, TITLE, SAMPLE, AUTHOR, TOPIC, CREATED } field;
+    enum { NONE, KEYWORDS, TITLE, SAMPLE, AUTHOR, CREATED } field;
   public:
     MetaXmlParser() : field(NONE), created(time_t(-1)) { }
     void process_text(const string &text);
     bool opening_tag(const string &tag);
     bool closing_tag(const string &tag);
-    string title, keywords, sample, author, topic;
+    string title, keywords, sample, author;
     time_t created;
+    int pages = -1;
 };
 
 #endif // OMEGA_INCLUDED_METAXMLPARSE_H
