@@ -1,7 +1,7 @@
 /** @file
  * @brief Run an external filter and capture its output in a std::string.
  */
-/* Copyright (C) 2003,2006,2007,2009,2010,2011,2013,2015,2017,2018 Olly Betts
+/* Copyright (C) 2003-2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include "safefcntl.h"
 #include <cerrno>
+#include <cinttypes>
 #include <cstdio>
 #include <cstring>
 #ifdef HAVE_SYS_TIME_H
@@ -38,9 +39,7 @@
 # include <sys/resource.h>
 #endif
 #include "safesysselect.h"
-#ifdef HAVE_SYS_SOCKET_H
-# include <sys/socket.h>
-#endif
+#include "safesyssocket.h"
 #include "safesyswait.h"
 #include "safeunistd.h"
 
@@ -52,14 +51,11 @@
 #include "setenv.h"
 #include "stringutils.h"
 
-#ifdef _MSC_VER
-# define popen _popen
-# define pclose _pclose
-#endif
-
 using namespace std;
 
+#ifndef __WIN32__
 static int devnull = -1;
+#endif
 
 #if defined HAVE_FORK && defined HAVE_SOCKETPAIR
 bool
@@ -234,6 +230,7 @@ void
 runfilter_init()
 {
     runfilter_init_signal_handlers_();
+#ifndef __WIN32__
     devnull = open("/dev/null", O_WRONLY);
     if (devnull < 0) {
 	cerr << "Failed to open /dev/null: " << strerror(errno) << endl;
@@ -246,6 +243,7 @@ runfilter_init()
     while (devnull <= 2) {
 	devnull = dup(devnull);
     }
+#endif
 }
 
 void
@@ -323,7 +321,7 @@ run_filter(int fd_in, const string& cmd, bool use_shell, string* out,
 	size_t j = 0;
 	while (true) {
 	    j = s.find_first_not_of(" \t\n", j);
-	    if (!(C_isalnum(s[j]) || s[j] == '_')) break;
+	    if (!(C_isalpha(s[j]) || s[j] == '_')) break;
 	    size_t i = j;
 	    do ++j; while (C_isalnum(s[j]) || s[j] == '_');
 	    if (s[j] != '=') {
@@ -456,21 +454,6 @@ run_filter(int fd_in, const string& cmd, bool use_shell, string* out,
 	    throw ReadError("wait pid failed");
     }
     pid_to_kill_on_signal = 0;
-#else
-    (void)use_shell;
-    FILE * fh = popen(cmd.c_str(), "r");
-    if (fh == NULL) throw ReadError("popen failed");
-    while (!feof(fh)) {
-	char buf[4096];
-	size_t len = fread(buf, 1, 4096, fh);
-	if (ferror(fh)) {
-	    (void)pclose(fh);
-	    throw ReadError("fread failed");
-	}
-	if (out) out->append(buf, len);
-    }
-    int status = pclose(fh);
-#endif
 
     if (WIFEXITED(status)) {
 	int exit_status = WEXITSTATUS(status);
@@ -479,10 +462,105 @@ run_filter(int fd_in, const string& cmd, bool use_shell, string* out,
 	if (exit_status == 127)
 	    throw NoSuchFilter();
     }
-#ifdef SIGXCPU
+# ifdef SIGXCPU
     if (WIFSIGNALED(status) && WTERMSIG(status) == SIGXCPU) {
 	cerr << "Filter process consumed too much CPU time" << endl;
     }
+# endif
+#else
+    (void)use_shell;
+    LARGE_INTEGER counter;
+    // QueryPerformanceCounter() will always succeed on XP and later
+    // and gives us a counter which increments each CPU clock cycle
+    // on modern hardware (Pentium or newer).
+    QueryPerformanceCounter(&counter);
+    char pipename[256];
+    snprintf(pipename, sizeof(pipename),
+	     "\\\\.\\pipe\\xapian-omega-filter-%lx-%lx_%" PRIx64,
+	     static_cast<unsigned long>(GetCurrentProcessId()),
+	     static_cast<unsigned long>(GetCurrentThreadId()),
+	     static_cast<unsigned long long>(counter.QuadPart));
+    pipename[sizeof(pipename) - 1] = '\0';
+    // Create a pipe so we can read stdout from the child process.
+    HANDLE hPipe = CreateNamedPipe(pipename,
+				   PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED,
+				   0,
+				   1, 4096, 4096, NMPWAIT_USE_DEFAULT_WAIT,
+				   NULL);
+
+    if (hPipe == INVALID_HANDLE_VALUE) {
+	throw ReadError("CreateNamedPipe failed");
+    }
+
+    HANDLE hClient = CreateFile(pipename,
+				GENERIC_READ|GENERIC_WRITE, 0, NULL,
+				OPEN_EXISTING,
+				FILE_FLAG_OVERLAPPED, NULL);
+
+    if (hClient == INVALID_HANDLE_VALUE) {
+	throw ReadError("CreateFile failed");
+    }
+
+    if (!ConnectNamedPipe(hPipe, NULL) &&
+	GetLastError() != ERROR_PIPE_CONNECTED) {
+	throw ReadError("ConnectNamedPipe failed");
+    }
+
+    // Set the appropriate handles to be inherited by the child process.
+    SetHandleInformation(hClient, HANDLE_FLAG_INHERIT, 1);
+
+    // Create the child process.
+    PROCESS_INFORMATION procinfo;
+    memset(&procinfo, 0, sizeof(PROCESS_INFORMATION));
+
+    STARTUPINFO startupinfo;
+    memset(&startupinfo, 0, sizeof(STARTUPINFO));
+    startupinfo.cb = sizeof(STARTUPINFO);
+    startupinfo.hStdError = GetStdHandle(STD_ERROR_HANDLE);
+    startupinfo.hStdOutput = hClient;
+    // FIXME: Is NULL the way to say "/dev/null"?
+    // It's what GetStdHandle() is documented to return if "an application does
+    // not have associated standard handles"...
+    startupinfo.hStdInput = fd_in >= 0 ? (HANDLE) _get_osfhandle(fd_in) : NULL;
+    startupinfo.dwFlags |= STARTF_USESTDHANDLES;
+
+    string cmdline{cmd};
+    // For some reason Windows wants a modifiable command line so we
+    // pass `&cmdline[0]` rather than `cmdline.c_str()`.
+    if (!CreateProcess(NULL, &cmdline[0],
+		       0, 0, TRUE, 0, 0, 0,
+		       &startupinfo, &procinfo)) {
+	if (GetLastError() == ERROR_FILE_NOT_FOUND)
+	    throw NoSuchFilter();
+	throw ReadError("CreateProcess failed");
+    }
+
+    CloseHandle(hClient);
+    CloseHandle(procinfo.hThread);
+    HANDLE child = procinfo.hProcess;
+
+    while (true) {
+	char buf[4096];
+	DWORD received;
+	if (!ReadFile(hPipe, buf, sizeof(buf), &received, NULL)) {
+	    throw ReadError("ReadFile failed");
+	}
+	if (received == 0) break;
+
+	if (out) out->append(buf, received);
+    }
+    CloseHandle(hPipe);
+
+    WaitForSingleObject(child, INFINITE);
+    DWORD rc;
+    while (GetExitCodeProcess(child, &rc) && rc == STILL_ACTIVE) {
+	Sleep(100);
+    }
+    CloseHandle(child);
+    int status = int(rc);
+    if (status == 0 || status == alt_status)
+	return;
+
 #endif
     throw ReadError(status);
 }
