@@ -4,7 +4,7 @@
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Sam Liddicott
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002-2022 Olly Betts
+ * Copyright 2002-2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,11 @@
 
 #include <config.h>
 
+#ifdef __CYGWIN__
+// Needed to get setenv() and strptime() declared.
+# define _GNU_SOURCE
+#endif
+
 #include <xapian.h>
 
 #include <algorithm>
@@ -275,6 +280,26 @@ report_useless_action(const string &file, size_t line, size_t pos,
     }
 }
 
+// Return true if we can support %z on the current platform.
+static inline bool
+parsedate_supports_z()
+{
+#ifndef HAVE_STRUCT_TM_TM_GMTOFF
+    // Without tm_gmtoff we aren't going to get the timezone information from
+    // strptime().
+    return false;
+#else
+    // Perform a simple run-time test to check if %z is suitably supported.
+    static bool cached_result = ([]() {
+	struct tm tm;
+	memset(&tm, 0, sizeof(tm));
+	auto ret = strptime("+1245", "%z", &tm);
+	return ret && *ret == '\0' && tm.tm_gmtoff == (12 * 60 + 45) * 60;
+    })();
+    return cached_result;
+#endif
+}
+
 static bool index_spec_uses_unique = false;
 
 static map<string, vector<Action>> index_spec;
@@ -477,7 +502,13 @@ parse_index_script(const string &filename)
 	    }
 	    if (code == Action::BAD) {
 		report_location(DIAG_ERROR, filename, line_no, action_pos);
-		cerr << "Unknown index action '" << action << "'\n";
+		if (action.empty()) {
+		    i = find_if(i, s.end(), C_isspace);
+		    cerr << "Expected index action, found '"
+			 << string(s, j - s.begin(), i - j) << "'\n";
+		} else {
+		    cerr << "Unknown index action '" << action << "'\n";
+		}
 	    }
 	    auto i_after_action = i;
 	    i = find_if(i, s.end(), [](char ch) { return !C_isspace(ch); });
@@ -697,15 +728,13 @@ bad_hex_digit:
 			    cerr << "Parsing timezone names with %Z is not "
 				    "supported\n";
 			}
-#ifndef HAVE_STRUCT_TM_TM_GMTOFF
 			bad_code = val.find("%z");
-			if (bad_code != val.npos) {
+			if (bad_code != val.npos && !parsedate_supports_z()) {
 			    report_location(DIAG_ERROR, filename, line_no,
 					    j - s.begin() + bad_code);
 			    cerr << "Parsing timezone offsets with %z is not "
 				    "supported on this platform\n";
 			}
-#endif
 			actions.emplace_back(code, action_pos, val);
 			break;
 		    }
